import time
import json
import boto3
import hashlib
import logging

from airflow.providers.amazon.aws.hooks.base_aws import AwsGenericHook as AwsHook
from airflow.providers.amazon.aws.waiters.base_waiter import BaseBotoWaiter


def get_efs_filesystem(aws_conn_id, identifier, region = None, **context):
    """
    Get EFS filesystem by identifier.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: EFS filesystem identifier
    :type identifier: str
    :return: EFS filesystem
    :rtype: dict
    """
    hook = AwsHook(aws_conn_id, client_type='efs', region_name=region)
    client = hook.get_client_type()

    filesystems = client.describe_file_systems(
        CreationToken=str(identifier)
    )

    if len(filesystems['FileSystems']) > 0:
        return filesystems['FileSystems'][0]
    else:
        return None
    
def create_efs_filesystem(aws_conn_id, identifier, args, region = None, **context):
    """
    Create EFS filesystem by identifier.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: EFS filesystem identifier
    :type identifier: str
    :param subnets: Subnets to create mount targets in
    :type subnets: list
    :param security_groups: Security groups to assign to mount targets
    :type security_groups: list
    :return: EFS filesystem
    :rtype: dict
    """

    hook = AwsHook(aws_conn_id, client_type='efs', region_name=region)
    client = hook.get_client_type()

    filesystems = client.describe_file_systems(
        CreationToken=str(identifier)
    )

    if len(filesystems['FileSystems']) > 0:
        filesystem = filesystems['FileSystems'][0]
        logging.info('Filesystem already exists: %s', filesystem)
    else:
        logging.info('Filesystem does not exist yet, creating...')
        filesystem = client.create_file_system(CreationToken=str(identifier), **args)
        logging.info('Filesystem created: %s', filesystem)

        waiter_config = {
            "version": 2,
            "waiters": {
                "EFSLifeCycleStateWaiter": {
                    "operation": "DescribeFileSystems",
                    "delay": 5,
                    "maxAttempts": 5,
                    "acceptors": [
                        {
                            "state": "success",
                            "matcher": "path",
                            "argument": f"length(FileSystems[?FileSystemId == '{filesystem['FileSystemId']}'] | [?"
                                            f"LifeCycleState == 'available'"
                                        "]) > `0`",
                            "expected": True
                        }
                    ]
                }
            }
        }
        base_waiter = BaseBotoWaiter(client, waiter_config)
        waiter = base_waiter.waiter('EFSLifeCycleStateWaiter')
        waiter.wait()

        logging.info('Filesystem available: %s', filesystem)

    return filesystem

def delete_efs_filesystem(aws_conn_id, identifier, region = None, **context):
    """
    Delete EFS filesystem by identifier.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: EFS filesystem identifier
    :type identifier: str
    :return: EFS filesystem
    :rtype: dict
    """

    hook = AwsHook(aws_conn_id, client_type='efs', region_name=region)
    client = hook.get_client_type()

    filesystems = client.describe_file_systems(
        CreationToken=str(identifier)
    )

    if len(filesystems['FileSystems']) > 0:
        filesystem = filesystems['FileSystems'][0]
        logging.info('Filesystem exists: %s', filesystem)
        client.delete_file_system(FileSystemId=filesystem['FileSystemId'])
        logging.info('Filesystem deleted: %s', filesystem)
    else:
        logging.info('Filesystem does not exist: %s', identifier)
        return None

    return filesystems

def create_mount_targets(aws_conn_id, identifier, subnets, security_groups, region = None, **context):
    """
    Create EFS mount targets by identifier.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: EFS filesystem identifier
    :type identifier: str
    :param subnets: Subnets to create mount targets in
    :type subnets: list
    :param security_groups: Security groups to assign to mount targets
    :type security_groups: list
    :return: EFS filesystem
    :rtype: dict
    """

    hook = AwsHook(aws_conn_id, client_type='efs', region_name=region)
    client = hook.get_client_type()

    filesystems = client.describe_file_systems(
        CreationToken=str(identifier)
    )

    if len(filesystems['FileSystems']) > 0:
        filesystem = filesystems['FileSystems'][0]
        logging.info('Filesystem exists: %s', filesystem)
    else:
        logging.info('Filesystem does not exist: %s', identifier)
        return None

    mount_targets = client.describe_mount_targets(
        FileSystemId=filesystem['FileSystemId']
    )

    # For each subnets, check if mount target exists and update security group if required
    for subnet in subnets:
        mount_target = next((x for x in mount_targets['MountTargets'] if x['SubnetId'] == subnet), None)

        if mount_target is not None:
            logging.info('Mount target already exists: %s', mount_target)
        else:
            logging.info('Mount target does not exist yet, creating...')
            mount_target = client.create_mount_target(FileSystemId=filesystem['FileSystemId'], SubnetId=subnet, SecurityGroups=security_groups)
            logging.info('Mount target created: %s', mount_target)

            waiter_config = {
                "version": 2,
                "waiters": {
                    "EFSMountTargetLifeCycleStateWaiter": {
                        "operation": "DescribeMountTargets",
                        "delay": 30,
                        "maxAttempts": 10,
                        "acceptors": [
                            {
                                "state": "success",
                                "matcher": "path",
                                "argument": f"length(MountTargets[?MountTargetId == '{mount_target['MountTargetId']}'] | [?"
                                                f"LifeCycleState == 'available'"
                                            "]) > `0`",
                                "expected": True
                            }
                        ]
                    }
                }
            }
            base_waiter = BaseBotoWaiter(client, waiter_config)
            waiter = base_waiter.waiter('EFSMountTargetLifeCycleStateWaiter')
            waiter.wait(MountTargetId=mount_target['MountTargetId'])

            logging.info('Mount target available: %s', mount_target)

            mount_targets = client.describe_mount_targets(
                FileSystemId=filesystem['FileSystemId']
            )

    return mount_targets

def delete_mount_targets(aws_conn_id, identifier, region = None, **context):
    """
    Delete EFS mount targets by identifier.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: EFS filesystem identifier
    :type identifier: str
    :return: EFS filesystem
    :rtype: dict
    """

    hook = AwsHook(aws_conn_id, client_type='efs', region_name=region)
    client = hook.get_client_type()

    filesystems = client.describe_file_systems(
        CreationToken=str(identifier)
    )

    if len(filesystems['FileSystems']) > 0:
        filesystem = filesystems['FileSystems'][0]
        logging.info('Filesystem exists: %s', filesystem)
    else:
        logging.info('Filesystem does not exist: %s', identifier)
        return None

    mount_targets = client.describe_mount_targets(
        FileSystemId=filesystem['FileSystemId']
    )

    # For each mount target, check if mount target exists and update security group if required
    for mount_target in mount_targets['MountTargets']:
        logging.info('Mount target exists: %s', mount_target)
        client.delete_mount_target(MountTargetId=mount_target['MountTargetId'])
        logging.info('Mount target deleted: %s', mount_target)

    waiter_config = {
        "version": 2,
        "waiters": {
            "EFSMountTargetLifeCycleStateWaiter": {
                "operation": "DescribeFileSystems",
                "delay": 30,
                "maxAttempts": 10 * len(mount_targets),
                "acceptors": [
                    {
                        "state": "success",
                        "matcher": "path",
                        "argument": f"length(FileSystems[?FileSystemId == '{filesystem['FileSystemId']}'] | [?"
                                        f"NumberOfMountTargets == `0`"
                                    "]) > `0`",
                        "expected": True
                    }
                ]
            }
        }
    }
    base_waiter = BaseBotoWaiter(client, waiter_config)
    waiter = base_waiter.waiter('EFSMountTargetLifeCycleStateWaiter')
    waiter.wait()

    return mount_targets


def put_efs_filesystem_policy(aws_conn_id, identifier, policy, region = None, **context):
    """
    Put EFS filesystem policy by identifier.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: EFS filesystem identifier
    :type identifier: str
    :param policy: EFS filesystem policy
    :type policy: str
    :return: EFS filesystem
    :rtype: dict
    """

    hook = AwsHook(aws_conn_id, client_type='efs', region_name=region)
    client = hook.get_client_type()

    filesystems = client.describe_file_systems(
        CreationToken=str(identifier)
    )

    if len(filesystems['FileSystems']) > 0:
        filesystem = filesystems['FileSystems'][0]
        logging.info('Filesystem exists: %s', filesystem)
    else:
        logging.info('Filesystem does not exist: %s', identifier)
        return None
    
    policy_string = json.dumps(policy)
    client.put_file_system_policy(FileSystemId=filesystem['FileSystemId'], Policy=policy_string)

    return filesystem


from airflow.hooks.base_hook import BaseHook
from airflow.contrib.operators.slack_webhook_operator import SlackWebhookOperator

def task_failed_slack_alert(context, slack_conn_id='slack_default', message=None):
    """
    Sends message to Slack about task failure.

    :param context: The execution context
    :type context: dict
    :param slack_conn_id: Slack connection ID
    :type slack_conn_id: str
    :return: None

    slack_conn_id: When specifying a custom connection for a specific channel,
    ensure the the connection is created as follow:
        - host: https://hooks.slack.com/services/
        - password: <webhook_token> (rest of the hook url)
    """

    slack_webhook_token = BaseHook.get_connection(slack_conn_id).password
    
    slack_msg = """
            :red_circle: Task Failed. 
            *Task*: {task}  
            *Dag*: {dag} 
            *Configuration*: {conf}
            *Message*: {message}
            *Execution Time*: {exec_date}  
            *Log Url*: {log_url} 
            """.format(
            task=context.get('task_instance').task_id,
            dag=context.get('task_instance').dag_id,
            ti=context.get('task_instance'),
            conf=context.get('task_instance').conf,
            exec_date=context.get('execution_date'),
            log_url=context.get('task_instance').log_url,
            message=message
        )
    
    failed_alert = SlackWebhookOperator(
        task_id='task_failed_slack_alert',
        http_conn_id=slack_conn_id,
        webhook_token=slack_webhook_token,
        message=slack_msg,
        username='airflow'
    )
    return failed_alert.execute(context=context)

def task_success_slack_alert(context, slack_conn_id='slack_default', message=None):
    """
    Sends message to Slack about task success.

    :param context: The execution context
    :type context: dict
    :param slack_conn_id: Slack connection ID
    :type slack_conn_id: str
    :return: None

    slack_conn_id: When specifying a custom connection for a specific channel,
    ensure the the connection is created as follow:
        - host: https://hooks.slack.com/services/
        - password: <webhook_token> (rest of the hook url)
    """

    slack_webhook_token = BaseHook.get_connection(slack_conn_id).password
    
    slack_msg = """
            :white_check_mark: Task Success. 
            *Task*: {task}  
            *Dag*: {dag} 
            *Configuration*: {conf}
            *Message*: {message}
            *Execution Time*: {exec_date}  
            *Log Url*: {log_url} 
            """.format(
            task=context.get('task_instance').task_id,
            dag=context.get('task_instance').dag_id,
            ti=context.get('task_instance'),
            conf=context.get('task_instance').conf,
            exec_date=context.get('execution_date'),
            log_url=context.get('task_instance').log_url,
            message=message
        )
    
    success_alert = SlackWebhookOperator(
        task_id='task_failed_slack_alert',
        http_conn_id=slack_conn_id,
        webhook_token=slack_webhook_token,
        message=slack_msg,
        username='airflow'
    )
    return success_alert.execute(context=context)


import time
import json
import boto3
import hashlib
import logging

from airflow.providers.amazon.aws.hooks.base_aws import AwsGenericHook as AwsHook
from airflow.providers.amazon.aws.waiters.base_waiter import BaseBotoWaiter

def get_task_definition(aws_conn_id, args, region = None, **context):
    """
    Get ECS task definition by identifier.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: ECS task definition identifier
    :type identifier: str
    :return: ECS task definition
    :rtype: dict
    """
    hook = AwsHook(aws_conn_id, client_type='ecs', region_name=region)
    client = hook.get_client_type()

    task_definitions = client.list_task_definitions(**args)

    if len(task_definitions['taskDefinitionArns']) > 0:
        task_definition_arn = task_definitions['taskDefinitionArns'][0]
        task_definition = client.describe_task_definition(
            taskDefinition=task_definition_arn
        )
        return task_definition['taskDefinition']
    else:
        return None
    
def get_task_definitions(aws_conn_id, args, region = None, **context):
    """
    Get ECS task definitions by familyPrefix.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param args: ECS task definition arguments
    :type args: dict
    :return: ECS task definition arns
    :rtype: list
    """

    hook = AwsHook(aws_conn_id, client_type='ecs', region_name=region)
    client = hook.get_client_type()

    task_definitions = client.list_task_definitions(**args)

    return task_definitions['taskDefinitionArns']
    
    
def register_task_definition(aws_conn_id, args, region = None, **context):
    """
    Register ECS task definition.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param args: ECS task definition arguments
    :type args: dict
    :return: ECS task definition
    :rtype: dict
    """
    hook = AwsHook(aws_conn_id, client_type='ecs', region_name=region)
    client = hook.get_client_type()

    args['cpu'] = str(args['cpu']) if 'cpu' in args else None
    args['memory'] = str(args['memory']) if 'memory' in args else None

    task_definition = client.register_task_definition(**args)

    return task_definition['taskDefinition']

def deregister_task_definition(aws_conn_id, args, region = None, **context):
    """
    Deregister ECS task definition.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: ECS task definition identifier
    :type identifier: str
    :return: ECS task definition
    :rtype: dict
    """
    hook = AwsHook(aws_conn_id, client_type='ecs', region_name=region)
    client = hook.get_client_type()

    task_definition = client.deregister_task_definition(**args)

    return task_definition['taskDefinition']

def deregister_task_definitions(aws_conn_id, args, region = None, **context):
    """
    Deregister ECS task definitions.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: ECS task definition identifier
    :type identifier: str
    :return: ECS task definition
    :rtype: dict
    """
    hook = AwsHook(aws_conn_id, client_type='ecs', region_name=region)
    client = hook.get_client_type()

    for task_definition in args['taskDefinitions']:
        client.deregister_task_definition(taskDefinition=task_definition)

    return args['taskDefinitions']

def delete_task_definitions(aws_conn_id, args, region = None, **context):
    """
    Delete ECS task definitions.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param identifier: ECS task definition identifier
    :type identifier: str
    :return: ECS task definition
    :rtype: dict
    """
    hook = AwsHook(aws_conn_id, client_type='ecs', region_name=region)
    client = hook.get_client_type()

    task_definition = client.delete_task_definitions(**args)
    return task_definition


from setuptools import setup, find_packages

VERSION = '0.0.9'
DESCRIPTION = 'Airflow utilities'
LONG_DESCRIPTION = 'A package that makes it easy to interact with AWS services from Airflow using PythonOperator'

setup(
    name="rosecape_airflow",
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    author="Nick Joanis",
    author_email="nicholas@rosecape.ca",
    license='MIT',
    packages=find_packages(),
    install_requires=[
        'boto3==1.26.142'
    ],
    keywords='Airflow',
    classifiers= [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        'License :: OSI Approved :: MIT License',
        "Programming Language :: Python :: 3",
    ]
)



import time
import json
import boto3
import hashlib
import logging

from airflow.providers.amazon.aws.hooks.base_aws import AwsGenericHook as AwsHook

def get_subnets(aws_conn_id, region, filters = [], **context):
    """
    Returns a list of subnet ids based on the filters provided.

    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param region: AWS region name
    :type region: str
    :param filters: Filters to apply when searching for subnets
    :type filters: list
    :return: List of subnet ids
    :rtype: list

    Example filters:
    filters = [
        {
            'Name': 'tag:Name',
            'Values': ['subnet-1', 'subnet-2']  
        }
    ]
    """
    
    hook = AwsHook(aws_conn_id, client_type='ec2', region_name=region)
    client = hook.get_client_type()

    response = client.describe_subnets(Filters=filters)
    return [subnet['SubnetId'] for subnet in response['Subnets']]

def get_security_groups(aws_conn_id, region, filters = [], **context):
    """
    Returns a list of security group ids based on the filters provided.
    
    :param aws_conn_id: AWS connection ID
    :type aws_conn_id: str
    :param region: AWS region name
    :type region: str
    :param filters: Filters to apply when searching for security groups
    :type filters: list
    :return: List of security group ids
    :rtype: list

    Example filters:
    filters = [
        {
            'Name': 'tag:Name',
            'Values': ['security-group-1', 'security-group-2']
        }
    ]
    """

    hook = AwsHook(aws_conn_id, client_type='ec2', region_name=region)
    client = hook.get_client_type()

    response = client.describe_security_groups(Filters=filters)
    return [security_group['GroupId'] for security_group in response['SecurityGroups']]